<h2>{{ resource.verbose_name|title }}</h2>
{% if resource.description %}
<p>{{ resource.description }}</p>
{% endif %}
{% for field in resource.fields %}
    {% include "odin/field.rst" %}
{% endfor %}
